#include <Wire.h>
#include <Adafruit_BNO08x.h>

// Create instance of BNO085
Adafruit_BNO08x bno08x;

// Structure to hold sensor data
sh2_SensorValue_t sensorValue;

void setup() {
  Serial.begin(115200);
  while (!Serial && millis() < 3000) delay(10);
  
  Serial.println("\nBNO085 Test Program");
  Serial.println("Initializing...");
  
  // Initialize I2C
  Wire.begin();
  Serial.println("I2C initialized");
  
  // Initialize BNO085
  if (!bno08x.begin_I2C()) {
    Serial.println("Failed to find BNO085 chip");
    while (1) { delay(10); }
  }
  Serial.println("BNO085 found!");
  
  // Set up the sensor for reading
  if (!bno08x.enableReport(SH2_ACCELEROMETER)) {
    Serial.println("Could not enable accelerometer");
  }
  if (!bno08x.enableReport(SH2_GYROSCOPE_CALIBRATED)) {
    Serial.println("Could not enable gyroscope");
  }
  if (!bno08x.enableReport(SH2_MAGNETIC_FIELD_CALIBRATED)) {
    Serial.println("Could not enable magnetometer");
  }
  if (!bno08x.enableReport(SH2_ROTATION_VECTOR)) {
    Serial.println("Could not enable rotation vector");
  }
  
  Serial.println("\nReading sensor data...");
  Serial.println("Format: Accel(x,y,z) Gyro(x,y,z) Mag(x,y,z) Quat(w,x,y,z)");
}

void loop() {
  if (bno08x.getSensorEvent(&sensorValue)) {
    switch (sensorValue.sensorId) {
      case SH2_ACCELEROMETER:
        Serial.print("Accel: ");
        Serial.print(sensorValue.un.accelerometer.x);
        Serial.print(", ");
        Serial.print(sensorValue.un.accelerometer.y);
        Serial.print(", ");
        Serial.print(sensorValue.un.accelerometer.z);
        Serial.println(" m/s²");
        break;
        
      case SH2_GYROSCOPE_CALIBRATED:
        Serial.print("Gyro: ");
        Serial.print(sensorValue.un.gyroscope.x);
        Serial.print(", ");
        Serial.print(sensorValue.un.gyroscope.y);
        Serial.print(", ");
        Serial.print(sensorValue.un.gyroscope.z);
        Serial.println(" rad/s");
        break;
        
      case SH2_MAGNETIC_FIELD_CALIBRATED:
        Serial.print("Mag: ");
        Serial.print(sensorValue.un.magneticField.x);
        Serial.print(", ");
        Serial.print(sensorValue.un.magneticField.y);
        Serial.print(", ");
        Serial.print(sensorValue.un.magneticField.z);
        Serial.println(" uT");
        break;
        
      case SH2_ROTATION_VECTOR:
        Serial.print("Quat: ");
        Serial.print(sensorValue.un.rotationVector.real);
        Serial.print(", ");
        Serial.print(sensorValue.un.rotationVector.i);
        Serial.print(", ");
        Serial.print(sensorValue.un.rotationVector.j);
        Serial.print(", ");
        Serial.print(sensorValue.un.rotationVector.k);
        Serial.println();
        break;
    }
  }
  
  delay(100);  // Read at 10Hz
} 