#include <Wire.h>
#include <Adafruit_BNO08x.h>

// Create instance of BNO085
Adafruit_BNO08x bno08x;

// Structure to hold sensor data
sh2_SensorValue_t sensorValue;

// Variables to store the latest readings
float accel_x = 0, accel_y = 0, accel_z = 0;
float gyro_x = 0, gyro_y = 0, gyro_z = 0;
float mag_x = 0, mag_y = 0, mag_z = 0;
float quat_w = 0, quat_x = 0, quat_y = 0, quat_z = 0;

void setup() {
  Serial.begin(115200);
  while (!Serial && millis() < 3000) delay(10);
  
  Serial.println("\nBNO085 Test Program");
  Serial.println("Initializing...");
  
  // Initialize I2C
  Wire.begin();
  Serial.println("I2C initialized");
  
  // Initialize BNO085
  if (!bno08x.begin_I2C()) {
    Serial.println("Failed to find BNO085 chip");
    while (1) { delay(10); }
  }
  Serial.println("BNO085 found!");
  
  // Set up the sensor for reading
  if (!bno08x.enableReport(SH2_ACCELEROMETER)) {
    Serial.println("Could not enable accelerometer");
  }
  if (!bno08x.enableReport(SH2_GYROSCOPE_CALIBRATED)) {
    Serial.println("Could not enable gyroscope");
  }
  if (!bno08x.enableReport(SH2_MAGNETIC_FIELD_CALIBRATED)) {
    Serial.println("Could not enable magnetometer");
  }
  if (!bno08x.enableReport(SH2_ROTATION_VECTOR)) {
    Serial.println("Could not enable rotation vector");
  }
  
  Serial.println("\nReading sensor data...");
  // Print headers for Serial Plotter
  Serial.println("AccelX,AccelY,AccelZ,GyroX,GyroY,GyroZ,MagX,MagY,MagZ,QuatW,QuatX,QuatY,QuatZ");
}

void loop() {
  if (bno08x.getSensorEvent(&sensorValue)) {
    switch (sensorValue.sensorId) {
      case SH2_ACCELEROMETER:
        accel_x = sensorValue.un.accelerometer.x;
        accel_y = sensorValue.un.accelerometer.y;
        accel_z = sensorValue.un.accelerometer.z;
        break;
        
      case SH2_GYROSCOPE_CALIBRATED:
        gyro_x = sensorValue.un.gyroscope.x;
        gyro_y = sensorValue.un.gyroscope.y;
        gyro_z = sensorValue.un.gyroscope.z;
        break;
        
      case SH2_MAGNETIC_FIELD_CALIBRATED:
        mag_x = sensorValue.un.magneticField.x;
        mag_y = sensorValue.un.magneticField.y;
        mag_z = sensorValue.un.magneticField.z;
        break;
        
      case SH2_ROTATION_VECTOR:
        quat_w = sensorValue.un.rotationVector.real;
        quat_x = sensorValue.un.rotationVector.i;
        quat_y = sensorValue.un.rotationVector.j;
        quat_z = sensorValue.un.rotationVector.k;
        break;
    }
  }
  
  // Print all values in a format suitable for Serial Plotter
  Serial.print(accel_x); Serial.print(",");
  Serial.print(accel_y); Serial.print(",");
  Serial.print(accel_z); Serial.print(",");
  Serial.print(gyro_x); Serial.print(",");
  Serial.print(gyro_y); Serial.print(",");
  Serial.print(gyro_z); Serial.print(",");
  Serial.print(mag_x); Serial.print(",");
  Serial.print(mag_y); Serial.print(",");
  Serial.print(mag_z); Serial.print(",");
  Serial.print(quat_w); Serial.print(",");
  Serial.print(quat_x); Serial.print(",");
  Serial.print(quat_y); Serial.print(",");
  Serial.println(quat_z);
  
  delay(50);  // Read at 20Hz for smoother plotting
} 